"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @ author: richen
 * @ copyright: Copyright (c) - <richenlin(at)gmail.com>
 * @ license: MIT
 * @ version: 2020-02-27 10:36:16
 */
const liteq = require('liteq');
const helper = liteq.helper;
class BaseModel extends liteq {
    /**
     * Creates an instance of BaseModel.
     * @param {...any[]} args
     * @memberof BaseModel
     */
    constructor(...args) {
        super(...args);
    }
    /**
     * Get table name
     *
     * @returns
     * @memberof BaseModel
     */
    getTableName() {
        return super.getTableName();
    }
    /**
     * Get the primary key
     *
     * @returns {string}
     * @memberof BaseModel
     */
    getPk() {
        return super.getPk();
    }
    /**
     * Get repository handle
     *
     * @param {boolean} [forceNew=false]
     * @returns
     * @memberof BaseModel
     */
    async getInstance(forceNew = false) {
        return super.getInstance(forceNew);
    }
    /**
     * Set repository handle to model
     *
     * @param {AdapterInstance} ins
     * @returns
     * @memberof BaseModel
     */
    setInstance(ins) {
        return super.setInstance(ins);
    }
    /**
     * Querying column names
     *
     * field(['aaa', 'bbb', 'ccc'])
     * @param {(string | string[])} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    field(values) {
        return super.field(values);
    }
    /**
     * Querying table name alias
     *
     * @param {string} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    alias(values) {
        return super.alias(values);
    }
    /**
     * Query conditions
     *
     * or:  where({or: [{...}, {...}]})
     *
     * not: where({not: {name: '', id: 1}})
     *
     * notin: where({notin: {'id': [1,2,3]}})
     *
     * null: where({id: null})
     *
     * notnull: where({id: {"<>": null}})
     *
     * in: where({id: [1,2,3]})
     *
     * and: where({id: 1, name: 'a'},)
     *
     * operator: where({id: {'<>': 1}})
     *
     * operator: where({id: {'<>': 1, '>=': 0, '<': 100, '<=': 10}})
     *
     * like: where({name: {'like': '%a'}})
     *
     * @param {Object} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    where(values) {
        return super.where(values);
    }
    /**
     * Querying limit
     *
     * limit(1)
     *
     * limit(10, 20)
     *
     * @param {number} skip
     * @param {number} [limit]
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    limit(skip, limit) {
        return super.limit(skip, limit);
    }
    /**
     * Querying order column
     *
     * order({xxx: 'desc'})
     *
     * @param {Object} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    order(values) {
        return super.order(values);
    }
    /**
     * Querying distinct columns
     *
     * distinct(['first_name'])
     *
     * @param {string[]} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    distinct(values) {
        return super.distinct(values);
    }
    /**
     * Querying group columns
     *
     * group('xxx')
     *
     * group(['xxx', 'xxx'])
     *
     * @param {(string | string[])} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    group(values) {
        return super.group(values);
    }
    /**
     * HAVING clause
     *
     * having({"name":{">": 100}})
     *
     * @param {Object} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    having(values) {
        return super.having(values);
    }
    /**
     * Join Querying
     *
     * join([{from: 'Test', alias: 'test', on: {aaa: bbb, ccc: ddd}, field: ['id', 'name'], type: 'inner'}])
     *
     * join([{from: 'Test', alias: 'test', on: {or: [{aaa: bbb}, {ccc: ddd}]}, field: ['id', 'name'], type: 'left'}])
     *
     * join([{from: 'Test', alias: 'test', on: {aaa: bbb, ccc: ddd}, field: ['id', 'name'], type: 'right'}])
     *
     * @param {any[]} values
     * @returns {BaseModel}
     * @memberof BaseModel
     */
    join(values) {
        return super.join(values);
    }
    /**
     * Pre-Add method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<Object>}
     * @memberof BaseModel
     */
    async _beforeAdd(data, options) {
        return data;
    }
    /**
     * Add method
     *
     * @param {Object | any[]} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async add(data, options) {
        try {
            if (helper.isEmpty(data)) {
                throw Error('Data can not be empty');
            }
            options = helper.parseOptions(this, options);
            const _data = await this._beforeAdd(data, options) || data;
            if (helper.isEmpty(_data)) {
                throw Error('Data can not be empty');
            }
            const result = await super.add(_data, options);
            //add return pk to data
            if (result) {
                Reflect.set(_data, this.pk, result);
            }
            await this._afterAdd(_data, options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Added after method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async _afterAdd(data, options) {
        return data;
    }
    /**
     * Added when query result is empty
     *
     * @param {Object} data
     * @param {*} [options={ where: {} }]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async thenAdd(data, options = { where: {} }) {
        try {
            if (helper.isEmpty(options) || helper.isEmpty(options.where)) {
                options.where = data;
            }
            const record = await this.find(options);
            if (helper.isEmpty(record)) {
                return this.add(data, options);
            }
            // tslint:disable-next-line: no-null-keyword
            return null;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Pre-Delete method
     *
     * @param {Object} options
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async _beforeDelete(options) {
        return options;
    }
    /**
     * Delete method
     *
     * @param {*} [options={}]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async delete(options = {}) {
        try {
            options = helper.parseOptions(this, options);
            await this._beforeDelete(options);
            const result = await super.delete(options);
            await this._afterDelete(options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Delete after method
     *
     * @param {Object} options
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async _afterDelete(options) {
        return options;
    }
    /**
     * Pre-Update method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<Object>}
     * @memberof BaseModel
     */
    async _beforeUpdate(data, options) {
        return data;
    }
    /**
     * Update method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async update(data, options) {
        try {
            options = helper.parseOptions(this, options);
            const _data = await this._beforeUpdate(data, options) || data;
            if (helper.isEmpty(_data)) {
                throw Error('Data can not be empty');
            }
            const result = await super.update(_data, options);
            await this._afterUpdate(_data, options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Increment method
     *
     * @param {string} field
     * @param {number} [step=1]
     * @param {*} [data={}]
     * @param {Object} [options]
     * @returns
     * @memberof BaseModel
     */
    async increment(field, step = 1, data = {}, options) {
        try {
            options = helper.parseOptions(this, options);
            const _data = await this._beforeUpdate(data, options) || data;
            const result = await super.increment(field, step, _data, options);
            await this._afterUpdate(_data, options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Decrement method
     *
     * @param {string} field
     * @param {number} [step=1]
     * @param {*} [data={}]
     * @param {Object} [options]
     * @returns
     * @memberof BaseModel
     */
    async decrement(field, step = 1, data = {}, options) {
        try {
            options = helper.parseOptions(this, options);
            const _data = await this._beforeUpdate(data, options) || data;
            const result = await super.decrement(field, step, _data, options);
            await this._afterUpdate(_data, options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Update after method
     *
     * @param {Object} data
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async _afterUpdate(data, options) {
        return data;
    }
    /**
     * Find one
     *
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async find(options) {
        try {
            options = helper.parseOptions(this, options);
            let result = await super.find(options);
            result = await this._afterFind(result, options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Find one after method
     *
     * @param {*} result
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _afterFind(result, options) {
        return result;
    }
    /**
     * Find all
     *
     * @param {Object} [options]
     * @returns {Promise<any[]>}
     * @memberof BaseModel
     */
    async select(options) {
        try {
            options = helper.parseOptions(this, options);
            let result = await super.select(options);
            result = await this._afterSelect(result, options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Paging query
     *
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async countSelect(options) {
        try {
            options = helper.parseOptions(this, options);
            const result = await super.countSelect(options);
            result.data = await this._afterSelect(result.data, options);
            return result;
        }
        catch (e) {
            return this.error(e);
        }
    }
    /**
     * Find all after method
     *
     * @param {*} result
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    _afterSelect(result, options) {
        return Promise.resolve(result);
    }
    /**
     * Count query
     *
     * @param {string} [field]
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async count(field, options) {
        return super.count(field, options);
    }
    /**
     * Sum query
     *
     * @param {string} field
     * @param {Object} [options]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async sum(field, options) {
        return super.sum(field, options);
    }
    /**
     * Native statement query
     *
     * mysql/posgre  TestModel.query('select ?, ? from test where id=?', ['id', 'name', 1]);
     *
     * mongo  TestModel.query('db.test.find()');
     *
     * @param {string} sqlStr
     * @param {any[]} [params=[]]
     * @returns {Promise<any[]>}
     * @memberof BaseModel
     */
    async query(sqlStr, params = []) {
        return super.query(sqlStr, params);
    }
    /**
     * Execute transaction
     *
     * @param {Function} func
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async transaction(func) {
        return super.transaction(func);
    }
    /**
     * Structure migration
     *
     * @param {string} [sqlStr]
     * @returns {Promise<any>}
     * @memberof BaseModel
     */
    async migrate(sqlStr) {
        return super.migrate(sqlStr);
    }
    /**
     * sqlString build
     *
     * {method: find | select | add | update | count | sum | decrement | increment}
     *
     * @param {string} [options={ method: "select" }]
     * @param {*} [data={}]
     * @returns {Promise<string>}
     * @memberof BaseModel
     */
    async sql(options = { method: "select" }, data = {}) {
        return super.sql(options, data);
    }
}
exports.BaseModel = BaseModel;
//# sourceMappingURL=BaseModel.js.map